"use client";
import DataTable from "@/components/tables/DataTable";
import { columnsDataBatches } from "../variables/columnsBatch";
import { TableData } from "@/variables/columnsData";
import AddButton from "@/components/buttons/add";
import { EditIcon, DeleteIcon } from "@chakra-ui/icons";
import { Flex, Button, Icon, useColorModeValue, Text } from "@chakra-ui/react";
import useShowToast from "@/app/hooks/useShowToast";
import { useRouter } from "next/navigation";
import { hasPermission } from "@/utils/navigation";
import { useGeneral } from "@/contexts/GeneralContext";
import { deleteBatch } from "@/app/lib/batches/action";
import { extractPagination } from "@/utils/common";

interface BatchesTableProps {
  batches: { data: TableData[] };
}

const BatchesTable: React.FC<BatchesTableProps> = ({ batches }) => {
  const { currentUser, batchStatuses } = useGeneral();
  const brandColor = useColorModeValue("brand.500", "white");
  const textColor = useColorModeValue("secondaryGray.900", "white");
  const router = useRouter();
  const showToast = useShowToast();

  const paginationData = extractPagination(batches);

  const handleAdd = () => {
    router.push("/admin/batches/create");
  };

  const handleEdit = (id: number) => {
    router.push(`/admin/batches/edit/${id}`);
  };

  const handleDelete = async (id: number) => {
    try {
      const response = await deleteBatch(id);
      if (response.status === false) {
        throw new Error(response.message || "Failed to delete batch.");
      }
      showToast("Batch Deleted!", response.message, "success");
      router.refresh();
    } catch (error: any) {
      showToast("Failed to delete", error.message, "error");
    }
  };
  const actions = (id: number) => (
    <Flex gap={2}>
      {hasPermission(currentUser.permissions, "batch", "edit") && (
        <Button
          variant="blank"
          alignItems="center"
          justifyContent="center"
          w="24px"
          h="24px"
          lineHeight="100%"
          borderRadius="10px"
          minW={0}
          onClick={() => {
            handleEdit(id);
          }}
          isDisabled={!hasPermission(currentUser.permissions, "batch", "edit")}
        >
          <Icon as={EditIcon} color={brandColor} w="20px" h="20px" />
        </Button>
      )}
      {hasPermission(currentUser.permissions, "batch", "delete") && (
        <Button
          variant="blank"
          alignItems="center"
          justifyContent="center"
          w="24px"
          h="24px"
          lineHeight="100%"
          borderRadius="10px"
          minW={0}
          onClick={() => {
            handleDelete(id);
          }}
          isDisabled={!hasPermission(currentUser.permissions, "batch", "delete")}
        >
          <Icon as={DeleteIcon} color={brandColor} w="20px" h="20px" />
        </Button>
      )}
    </Flex>
  );

  const cellRenderer = (cell: any) => {
    if (cell.column.id === "status") {
      return (
        <Text color={textColor} fontSize="sm" fontWeight="700">
          {batchStatuses[cell.value]}
        </Text>
      );
    }
    return (
      <Text color={textColor} fontSize="sm" fontWeight="700">
        {cell.value}
      </Text>
    );
  };

  return (
    <DataTable
      columnsData={columnsDataBatches}
      tableData={batches.data as unknown as TableData[]}
      isSelectable={false}
      showPageSize={false}
      paginate={paginationData}
      addButton={hasPermission(currentUser.permissions, "batch", "create") &&
        <AddButton
          onClick={handleAdd}
          isDisabled={
            !hasPermission(currentUser.permissions, "batch", "create")
          }
        />
      }
      actionSlot={actions}
      cellRenderer={cellRenderer}
    />
  );
};

export default BatchesTable;
